/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2024 NVIDIA CORPORATION & AFFILIATES
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __gb10b_dev_fault_h__
#define __gb10b_dev_fault_h__

#define NV_PFAULT_CLIENT_HUB_PD1         0x0000001E /*       */
#define NV_PFAULT_CLIENT_HUB_SSYNC1      0x00000028 /*       */
#define NV_PFAULT_CLIENT_HUB_SSYNC2      0x00000029 /*       */
#define NV_PFAULT_CLIENT_HUB_SSYNC3      0x00000036 /*       */
#define NV_PFAULT_CLIENT_HUB_SCC1        0x00000040 /*       */
#define NV_PFAULT_CLIENT_HUB_SCC_NB1     0x00000041 /*       */
#define NV_PFAULT_CLIENT_HUB_SCC2        0x00000042 /*       */
#define NV_PFAULT_CLIENT_HUB_SCC_NB2     0x00000043 /*       */
#define NV_PFAULT_CLIENT_HUB_SCC3        0x00000044 /*       */
#define NV_PFAULT_CLIENT_HUB_SCC_NB3     0x00000045 /*       */
#define NV_PFAULT_CLIENT_HUB_RASTERTWOD1 0x00000046 /*       */
#define NV_PFAULT_CLIENT_HUB_RASTERTWOD2 0x00000047 /*       */
#define NV_PFAULT_CLIENT_HUB_RASTERTWOD3 0x00000048 /*       */
#define NV_PFAULT_CLIENT_HUB_GSPLITE1    0x00000049 /*       */
#define NV_PFAULT_CLIENT_HUB_GSPLITE2    0x0000004A /*       */
#define NV_PFAULT_CLIENT_HUB_GSPLITE3    0x0000004B /*       */
#define NV_PFAULT_CLIENT_HUB_PD2         0x0000004C /*       */
#define NV_PFAULT_CLIENT_HUB_PD3         0x0000004D /*       */
#define NV_PFAULT_CLIENT_HUB_ESC0         0x00000063 /*       */
#define NV_PFAULT_CLIENT_HUB_ESC1         0x00000064 /*       */
#define NV_PFAULT_CLIENT_HUB_ESC2         0x00000065 /*       */
#define NV_PFAULT_CLIENT_HUB_ESC3         0x00000066 /*       */
#define NV_PFAULT_CLIENT_HUB_ESC4         0x00000067 /*       */
#define NV_PFAULT_CLIENT_HUB_ESC5         0x00000068 /*       */
#define NV_PFAULT_CLIENT_HUB_ESC6         0x00000069 /*       */
#define NV_PFAULT_CLIENT_HUB_ESC7         0x0000006a /*       */
#define NV_PFAULT_CLIENT_HUB_ESC8         0x0000006b /*       */
#define NV_PFAULT_CLIENT_HUB_ESC9         0x0000006c /*       */
#define NV_PFAULT_CLIENT_HUB_ESC10        0x0000006d /*       */
#define NV_PFAULT_CLIENT_HUB_ESC11        0x0000006e /*       */
#define NV_PFAULT_CLIENT_HUB_GSPLITE     0x0000007B /*       */
#define NV_PFAULT_CLIENT_HUB_GSPLITE0    0x0000007B /*       */
#define NV_PFAULT_CLIENT_HUB_VPR_SCRUBBER2 0x0000007C /*       */
#define NV_PFAULT_CLIENT_HUB_VPR_SCRUBBER3 0x0000007D /*       */
#define NV_PFAULT_CLIENT_HUB_VPR_SCRUBBER4 0x0000007E /*       */
#define NV_PFAULT_CLIENT_HUB_NVENC3      0x0000007F /*       */

#endif // __gb10b_dev_fault_h__
